/** \file OSkA.h
 * \brief This file contains all OSkA class definitions.
 *
 * Here you find definitions for all available OSkA classes. So in your header files
 * you can just include the OSkA.h and have all classes available.
 */

#pragma once

namespace OSkA
{
	class Shader;
	class Shader;
	class AnimationShader;
	class ShaderInterface;

	class Animation;
	class Bone;
	class Keyframe;
	class Skeleton;
	class Track;

	class Matrix;

	class Mesh;
	class VertexBones;

	class AnimationCooker;
	class BoneCooker;
}
